#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import codecs
import os
import os.path
import pytest
import stat
import subprocess
import sys

if sys.version_info.major == 2:
    from StringIO import StringIO
else:
    from io import StringIO

import x2y.options
import x2y.x2y

class Cmd(object):
    def __init__(self, cmd, chdir='', indir='', stdin=None, stdout=None, stderr=None, exitcode=0, reference=None):
        self.cmd = cmd
        self.chdir = chdir
        self.indir = indir
        self.reference = reference
        # set and rewind stdin
        self.stdin = stdin
        if stdin:
            self.stdin = StringIO()
            for f in stdin:
                self.stdin.write(f + '\n')
            self.stdin.seek(0)
        self.stdout = stdout
        if stdout:
            if chdir:
                self.stdout = []
                for o in stdout:
                    self.stdout.append(os.path.join(chdir, o))
            elif indir:
                self.stdout = []
                for o in stdout:
                    self.stdout.append(os.path.join(indir, o))
        self.stderr = stderr
        self.exitcode = exitcode

    def __str__(self):
        return self.cmd

    def argv(self):
        args = []
        if self.chdir:
            args.extend(['-d', self.chdir])
        args.extend(self.cmd.split())
        return args

    def cmdline(self):
        args = ['x2y']
        args.extend(self.argv())
        return args

    def run(self):
        rval = Cmd(self.cmd)
        argv = self.argv()
        stdout = StringIO()
        stderr = StringIO()
        rval.exitcode = x2y.x2y.main(argv, stdin=self.stdin, stdout=stdout, stderr=stderr)
        stdout_value = stdout.getvalue()
        stderr_value = stderr.getvalue()
        if stdout_value:
            rval.stdout = stdout_value.strip().split('\n')
        if stderr_value:
            rval.stderr = stderr_value.strip().split('\n')
        stdout.close()
        stderr.close()
        if self.stdin:
            self.stdin.close()
        return rval

    def run_as_process(self):
        rval = Cmd(self.cmd)
        try:
            cmd = self.cmdline()
            os.environ['COVERAGE_PROCESS_START'] = '1'
            env = os.environ.copy()
            env['COVERAGE_FILE'] = '.coverage.%s' % (self.cmd.replace('/', '-').replace(' ', '-'))
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
        except Exception as e:
            pytest.fail(msg='Cmd: run: exception running %s: %s' % (cmd, e))
        else:
            stdout, stderr = p.communicate()
            if stdout:
                rval.stdout = stdout.decode().strip().split('\n')
            if stderr:
                rval.stderr = stderr.decode().strip().split('\n')
            rval.exitcode = p.wait()
        return rval

    def outfile(self):
        argv = self.argv()
        i = 0
        while i < len(argv):
            if argv[i] == '-o':
                return argv[i + 1]
        return None
    
    def files_match(self):
        outfile = self.outfile()
        with open(outfile, 'rb') as fp:
            out_bytes = bytearray(fp.read())

        with open(self.reference, 'rb') as fp:
            ref_bytes = bytearray(fp.read())
        
        if out_bytes == ref_bytes:
            return True
        pyf.fail(msg='Files differ: %s %s' % (self.reference, outfile))
        return False


some_bad_option = '--some-bad-option'
usage_string_expanded = 'usage: %s' % (x2y.options.usage_string % {'prog': x2y.options.program_name})
some_bad_option_error_msg = '%(prog)s: error: unrecognized arguments: %(option)s' % {'prog': x2y.options.program_name, 'option': some_bad_option}


cmds = [
    # DOS
    Cmd('-o tests-out/mac.txt -f dos -t mac tests/in/dos.txt', reference='tests/in/mac.txt'),
    Cmd('-o tests-out/unx.txt -f dos -t unx tests/in/dos.txt', reference='tests/in/unx.txt'),
    
    # Mac
    Cmd('-o tests-out/mac.txt -f mac -t dos tests/in/mac.txt', reference='tests/in/dos.txt'),
    Cmd('-o tests-out/dos.txt -f mac -t unx tests/in/mac.txt', reference='tests/in/unx.txt'),
    
    # Unix
    Cmd('-o tests-out/dos.txt -f unx -t dos tests/in/unx.txt', reference='tests/in/dos.txt'),
    Cmd('-o tests-out/mac.txt -f unx -t mac tests/in/unx.txt', reference='tests/in/mac.txt'),
]

class TestCmd(object):
    @pytest.mark.parametrize('cmd', cmds)
    def test_cmd(self, cmd):
        r = cmd.run()
        assert r.stderr == cmd.stderr
        assert r.stdout == cmd.stdout
        assert r.exitcode == cmd.exitcode
        assert cmd.files_match()


if __name__ == '__main__':
    pytest.main()





# -*- coding: utf-8 -*-
from __future__ import print_function

import datetime
import logging
import logging.handlers
import os

# logging

# the logger
glogger = None

# init default root object logging
logging.basicConfig(level=logging.DEBUG, format='%(name)s %(levelname)s %(message)s', filename=os.devnull)


class FileFormatter(logging.Formatter):
    def formatTime(self, record, datefmt=None):
        dt = datetime.datetime.utcfromtimestamp(record.created)
        if datefmt:
            s = dt.strftime(datefmt)
        else:
            s = dt.strftime('%Y-%m-%d %H:%M:%S %Z')
        return s


def open_logging_console():
    log_console = logging.StreamHandler()
    log_console.setLevel(logging.DEBUG)
    formatter = logging.Formatter('%(name)s %(levelname)s %(message)s')
    log_console.setFormatter(formatter)
    return log_console


def open_logging_file(fname):
    # make sure the destination directory exists
    try:
        os.makedirs(os.path.dirname(fname))
    except:
        pass
    log_file = logging.handlers.TimedRotatingFileHandler(fname, when='midnight', backupCount=10, encoding='utf8')
    log_file.setLevel(logging.DEBUG)
    formatter = FileFormatter('%(asctime)s %(name)s %(levelname)s %(message)s', datefmt='%Y-%m-%d %H:%M:%S %Z')
    log_file.setFormatter(formatter)
    return log_file


def init_logging(options):
    if not options.debug:
        return

    global glogger

    glogger = logging.getLogger('x2y')

    # console
    try:
        l = open_logging_console()
    except Exception as e:
        print('Exception initing logging to console: %s' % e)
    else:
        glogger.addHandler(l)

    # file
    if options.debug_log:
        try:
            l = open_logging_file(options.debug_log)
        except Exception as e:
            print('Exception initing logging to file %s: %s' % (options.debug_log, e))
        else:
            glogger.addHandler(l)


def deinit_logging():
    logging.shutdown()


# catches potential logging errors
def do_log(msg, which, **kwargs):
    global glogger

    if not glogger:
        return

    try:
        func = getattr(glogger, which, None)
        if func and callable(func):
            func(msg.encode('unicode_escape', errors='replace'), **kwargs)
        else:
            pass
            #print('do_log: no function: %s' % which)
    except Exception:
        pass
#    except Exception as e:
#        print('do_log: exception: %s' % e)


def debug(msg, **kwargs):
    do_log(msg, 'debug', **kwargs)


def info(msg, **kwargs):
    do_log(msg, 'info', **kwargs)


def warning(msg, **kwargs):
    do_log(msg, 'warning', **kwargs)


def error(msg, **kwargs):
    do_log(msg, 'error', **kwargs)


def critical(msg, **kwargs):
    do_log(msg, 'critical', **kwargs)


def exception(msg, **kwargs):
    do_log(msg, 'exception', **kwargs)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# x2y: convert line ending between DOS, Mac and Unix
# https://github.com/bnomis/x2y
# (c) Simon Blanchard

import os
import os.path
import sys


from .logger import error, init_logging


def writerr(options, line, exception=None, set_exit_status=True):
    if set_exit_status:
        options.exit_status = 'error'
    options.stderr.write(line + '\n')
    if exception:
        error(line, exc_info=True)


def writerr_file_access(options, line, exception=None):
    if not options.suppress_file_access_errors:
        writerr(options, line, exception=exception, set_exit_status=False)


def check_file_access(options, path):
    # check file exists
    # will return true even for broken symlinks
    if not os.path.lexists(path):
        writerr_file_access(options, 'File does not exist: %s' % path)
        return False

    # double check for broken symlinks
    if os.path.islink(path):
        if not os.path.exists(path):
            writerr_file_access(options, 'Broken symlink: %s' % path)
            return False

    # check can open for read
    if os.path.isdir(path):
        if not os.access(path, os.R_OK):
            writerr_file_access(options, 'Directory is not readable: %s' % path)
            return False
    else:
        try:
            fp = open(path)
        except Exception as e:
            writerr_file_access(options, 'File is not readable: %s' % path)
            error('check_file_access: exception for %s: %s' % (path, e), exc_info=True)
            return False
        else:
            fp.close()

    return True


def make_outdir(options):
    if options.directory:
        if not os.path.exists(options.directory):
            path = ''
            for p in options.directory.split('/'):
                path = os.path.join(path, p)
                if not os.path.exists(path):
                    try:
                        os.mkdir(path)
                    except Exception as e:
                        writerr(options, 'Exception making directory: %s' % path, exception=e)


def output_filename(options, filename):
    if options.output:
        return options.output

    if options.directory:
        make_outdir(options)
        base = os.path.basename(filename)
        return os.path.join(options.directory, base)

    return filename


def backup(options, path):
    toname = path + '.' + options.backup
    os.rename(path, toname)


def x2y_bytes(options, in_bytes):
    start = pos = 0
    from_length = len(options.from_bytes)
    out_bytes = bytearray()
    in_length = len(in_bytes)
    while start < in_length:
        pos = in_bytes.find(options.from_bytes, start)
        
        if pos == -1:
            break
        out_bytes.extend(in_bytes[start:pos])
        out_bytes.extend(options.to_bytes)
        line_length = pos - start + from_length
        start += line_length
    if start < in_length - 1:
        out_bytes.extend(in_bytes[start:])
    return out_bytes


def x2y_file(options, path):
    if not check_file_access(options, path):
        return

    with open(path, 'rb') as fp:
        in_bytes = bytearray(fp.read())

    out_bytes = x2y_bytes(options, in_bytes)

    if in_bytes != out_bytes:
        options.did_convert = True

        outname = output_filename(options, path)
        if outname == path and options.backup:
            backup(options, path)

        with open(outname, 'wb') as fp:
            fp.write(out_bytes)


def x2y(options):
    for f in options.files:
        x2y_file(options, f)


def main(argv, stdin=None, stdout=None, stderr=None):
    from .options import parse_opts
    exit_statuses = {
        'converted': 0,
        'no-convert': 1,
        'error': 2,
        'not-set': -1
    }
    line_endings = {
        'dos': u'\r\n',
        'mac': u'\r',
        'unx': u'\n'
    }

    options = parse_opts(argv, stdin=stdin, stdout=stdout, stderr=stderr)
    if not options:
        return exit_statuses['error']

    # convert from/to same thing does nothing
    if options.from_end == options.to_end:
        return exit_statuses['no-convert']

    options.from_bytes = bytearray(line_endings[options.from_end], 'utf8')
    options.to_bytes = bytearray(line_endings[options.to_end], 'utf8')

    init_logging(options)

    # do the conversions
    try:
        x2y(options)
    except KeyboardInterrupt:
        writerr(options, '\nInterrupted')
    except Exception as e:
        writerr(options, 'x2y exception', exception=e)
        options.exit_status = 'error'

    if options.exit_status == 'not-set':
        if options.did_convert:
            options.exit_status = 'converted'
        else:
            options.exit_status = 'no-convert'

    return exit_statuses[options.exit_status]


def run():
    sys.exit(main(sys.argv[1:]))


if __name__ == '__main__':
    run()





# -*- coding: utf-8 -*-
from __future__ import print_function

import argparse
import sys

from . import __version__


# put these string here so we can import them for testing
program_name = 'x2y'
usage_string = '%(prog)s [options] --from line-ending --to line-ending File [File ...]'
version_string = '%(prog)s %(version)s' % {'prog': program_name, 'version': __version__}
description_string = '''x2y: convert line ending between DOS, Mac (pre OS X) and Unix'''


def parse_opts(argv, stdin=None, stdout=None, stderr=None):
    choices = [u'dos', u'mac', u'unx']

    parser = argparse.ArgumentParser(
        prog=program_name,
        usage=usage_string,
        description=description_string,
        formatter_class=argparse.RawDescriptionHelpFormatter
    )

    parser.add_argument(
        '--version',
        action='version',
        version=version_string
    )

    parser.add_argument(
        '--debug',
        dest='debug',
        action='store_true',
        default=False,
        help='Turn on debug logging.'
    )

    parser.add_argument(
        '--debug-log',
        dest='debug_log',
        metavar='FILE',
        help='Save debug logging to FILE.'
    )

    parser.add_argument(
        '-b',
        '--backup',
        metavar='BACKUP EXTENTSION',
        help='Save a backup of the input file by renaming with BACKUP EXTENTSION. Ignored if the -o option is given. Default: %(default)s.'
    )

    parser.add_argument(
        '-d',
        '--directory',
        metavar='DIRECTORY',
        help='Save extracted text to DIRECTORY. Ignored if the -o option is given.'
    )

    parser.add_argument(
        '-f',
        '--from',
        dest='from_end',
        choices=choices,
        required=True,
        help='Line ending to convert from.'
    )

    parser.add_argument(
        '-o',
        '--output',
        metavar='FILE',
        help='Save extracted text to FILE. If not given, the output file is named the same as the input file but with a txt extension. The extension can be changed with the -e option. Files are opened in append mode unless the -X option is given.'
    )

    parser.add_argument(
        '-t',
        '--to',
        dest='to_end',
        choices=choices,
        required=True,
        help='Line ending to convert to.'
    )

    parser.add_argument(
        '-A',
        '--suppress-file-access-errors',
        default=False,
        action='store_true',
        help='Do not print file/directory access errors.'
    )

    parser.add_argument(
        'files',
        metavar='File',
        nargs='+',
        help='Files to convert'
    )

    # print('argv = %s' % argv)
    options = parser.parse_args(argv)

    # set up i/o options
    options.stdin = stdin or sys.stdin
    options.stdout = stdout or sys.stdout
    options.stderr = stderr or sys.stderr

    options.did_convert = False
    options.exit_status = 'not-set'

    return options

